package com.supermap.sample.serviceprovider;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.imageio.ImageIO;

import org.apache.commons.lang3.StringUtils;

import com.supermap.services.components.commontypes.Layer;
import com.supermap.services.components.commontypes.MapParameter;
import com.supermap.services.components.commontypes.OutputFormat;
import com.supermap.services.components.commontypes.Point2D;
import com.supermap.services.components.commontypes.Rectangle;
import com.supermap.services.components.commontypes.Rectangle2D;
import com.supermap.services.components.commontypes.RectifyType;
import com.supermap.services.components.commontypes.Unit;
import com.supermap.services.providers.LocalTileProviderBase;
import com.supermap.services.util.PrjCoordSysConversionTool;
import com.supermap.services.util.TileTool;

/**
 * <p>
 * ArcGIS离散缓存  地图服务提供者。
 * </p>
 * <p>
 * ArcGISExplodedCacheMapProvider 提供了将本地ArcGIS离散缓存发布为地图服务的能力，并封装了与 SuperMap iServer 地图相关的 GIS
 * 功能。
 * </p>
 * <p>
 * ArcGIS disperse cache   Map service provider.
 * </p>
 * <p>
 * ArcGISExplodedCacheMapProvider supports to publish local ArcGIS disperse caches as map service, and encapsulates the map related GIS functions of SuperMap iServer.
 * </p>
 *
 */
public class ArcGISExplodedCacheMapProvider extends LocalTileProviderBase {
    private static final String MAPNAME = "arcgismaps";
    private double[] scales;
    private double[] resolutions = null;
    private Point2D origin;

    /**
     * <p>
     * 获取图片
     * </p>
     * <p>
     * Get the image
     * </p>
     * @param tileParam
     * @param x
     * @param y
     * @param resolution
     * @return
     */
    @Override
    protected BufferedImage getTileImage(TileImagePrameter paramTileImagePrameter) {
        ArcGISExplodedCacheMapProviderSetting setting = (ArcGISExplodedCacheMapProviderSetting) this.getMapProviderSetting();
        String arcgisMapsAdressTemplate = setting.getCachePath();
        arcgisMapsAdressTemplate += "/_alllayers/L%02d/R%08x/C%08x.png";
        int z = getLevelByResolution(paramTileImagePrameter.resolution);
        if (z < 0) {
            return TileTool.getBlankImage(this.tilePixWidth, this.tilePixHeight, paramTileImagePrameter.outputOption.transparent);
        }
        String arcgisImagePath = String.format(arcgisMapsAdressTemplate, z, paramTileImagePrameter.y, paramTileImagePrameter.x);
        if (!new File(arcgisImagePath).exists()) {
            return TileTool.getBlankImage(this.tilePixWidth, this.tilePixHeight, paramTileImagePrameter.outputOption.transparent);
        }
        BufferedImage image = null;
        FileInputStream fs = null;
        try {
            fs = new FileInputStream(arcgisImagePath);
            image = ImageIO.read(fs);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                fs.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return image;
    }

    /**
     * <p>
     * 初始化支持的地图列表。
     * </p>
     *
     * Initialize the supported map list.
     * @return
     *
     * 地图名列表
     * <p>
     * Map name list
     * </p>
     */
    @Override
    protected List<String> initSupportedMapNames() {
        List<String> names = new ArrayList<String>();
        names.add(getMapName());
        return names;
    }

    /**
     * <p>
     * 初始化默认地图状态，返回值为一个映射，键为地图名，值为地图参数对象。
     * </p>
     * <p>
     * Initialize the default map status. Return value is a mapping. The key is the map name and the value is the map parameter object.
     * </p>
     * @return 地图状态映射，键为地图名，值为地图参数对象。
     * <p>Map status mapping. The key is the map name, and the value is the parameter object.</p>
     */
    @Override
    protected Map<String, MapParameter> initDefaultMapParameter() {
        String mapName = getMapName();
        ArcGISExplodedCacheMapProviderSetting setting = (ArcGISExplodedCacheMapProviderSetting) this.getMapProviderSetting();
        if (StringUtils.isBlank(setting.getCachePath())) {
            throw new IllegalArgumentException("cahcePath is null");
        }
        ArcGISXMLParseTool arcgisXmlParseTool = new ArcGISXMLParseTool(getConfXMLPath());
        tilePixHeight = arcgisXmlParseTool.getTileRows();
        tilePixWidth = arcgisXmlParseTool.getTileCols();
        MapParameter defaultMapParameter = new MapParameter();
        defaultMapParameter.name = mapName;
        defaultMapParameter.prjCoordSys = PrjCoordSysConversionTool.getPrjCoordSys(arcgisXmlParseTool.getWkid());
        defaultMapParameter.coordUnit = defaultMapParameter.prjCoordSys.coordUnit;
        defaultMapParameter.distanceUnit = Unit.METER;
        defaultMapParameter.viewer = new Rectangle(0, 0, arcgisXmlParseTool.getTileCols(), arcgisXmlParseTool.getTileRows());
        defaultMapParameter.bounds = arcgisXmlParseTool.getBounds(getConfCDIPath());
        defaultMapParameter.center = defaultMapParameter.bounds.center();
        double[] resolutionArray = getSupportResolutions(mapName);
        defaultMapParameter.viewBounds = new Rectangle2D(defaultMapParameter.center.x - arcgisXmlParseTool.getTileCols() * resolutionArray[0] * 0.5,
                defaultMapParameter.center.y - arcgisXmlParseTool.getTileRows() * resolutionArray[0] * 0.5, defaultMapParameter.center.x
                        + arcgisXmlParseTool.getTileCols() * resolutionArray[0] * 0.5, defaultMapParameter.center.y + arcgisXmlParseTool.getTileRows()
                        * resolutionArray[0] * 0.5);
        defaultMapParameter.customEntireBoundsEnabled = false;
        defaultMapParameter.rectifyType = RectifyType.BYCENTERANDMAPSCALE;
        defaultMapParameter.visibleScalesEnabled = true;
        defaultMapParameter.visibleScales = getSupportScales();
        defaultMapParameter.scale = defaultMapParameter.visibleScales[0];
        defaultMapParameter.layers = new ArrayList<Layer>();
        ArcGISExplodedCacheMapLayer layer = new ArcGISExplodedCacheMapLayer();
        layer.name = defaultMapParameter.name;
        layer.bounds = new Rectangle2D(defaultMapParameter.bounds);
        layer.visible = true;
        defaultMapParameter.layers.add(layer);
        Map<String, MapParameter> result = new HashMap<String, MapParameter>();
        result.put(mapName, defaultMapParameter);
        return result;
    }

    /**
    *
    * <p>
    * 返回指定地图支持的图片格式数组
    * </p>
    * <p>
    * Return image format array supported by specified map
    * </p>
    * @param mapName
    * @return
    * @since 7.1.0
    */
    @Override
    public OutputFormat[] getSupportImageFormat(String paramString) {
        return new OutputFormat[0];
    }

    /**
     * <p>
     * 获取地图的左上角地理坐标。
     * </p>
     * <p>
     * Get the geographical coordinates of map top left corner.
     * </p>
     * @param 地图名
     * <p>
     * Map name
     * </p>
     * @return 地图的左上角地理坐标。
     * <p>
     * The geographical coordinates of map top left corner.
     * </p>
     */
    @Override
    protected Point2D getOrigin(String paramString) {
        ArcGISXMLParseTool arcgisXmlParseTool = new ArcGISXMLParseTool(getConfXMLPath());
        if (origin == null) {
            origin = arcgisXmlParseTool.getTileOrigin();
            return origin;
        }
        return origin;
    }

    @Override
    protected boolean cacheEnabled() {
        return false;
    }

    /**
     * <p>
     * 返回当前服务支持的分辨率 。如果支持所有分辨率，则返回空。
     * <p/>
     * <p>
     * Return the resolutions supported by current services. If all resolutions are supported, return null.
     * <p/>
     * @param mapName
     *
     * @return 分辨率数组
     * <p>
     * Resolution array
     * </p>
     */
    @Override
    public double[] getSupportResolutions(String paramString) {
        ArcGISXMLParseTool arcgisXmlParseTool = new ArcGISXMLParseTool(getConfXMLPath());
        if (resolutions == null) {
            resolutions = arcgisXmlParseTool.getResolutions();
        }
        return resolutions;
    }

    private int getLevelByResolution(double resolution) {
        ArcGISXMLParseTool arcgisXmlParseTool = new ArcGISXMLParseTool(getConfXMLPath());
        for (int i = 0; i < arcgisXmlParseTool.getResolutions().length; i++) {
            if (resolution == arcgisXmlParseTool.getResolutions()[i]) {
                return i;
            }
        }
        return -1;
    }

    private double[] getSupportScales() {
        ArcGISXMLParseTool arcgisXmlParseTool = new ArcGISXMLParseTool(getConfXMLPath());
        if (scales == null) {
            scales = arcgisXmlParseTool.getScales();
        }
        return scales;
    }

    private String getConfXMLPath() {
        ArcGISExplodedCacheMapProviderSetting setting = (ArcGISExplodedCacheMapProviderSetting) this.getMapProviderSetting();
        String path = setting.getCachePath() + "/conf.xml";
        return path;
    }

    private String getConfCDIPath() {
        ArcGISExplodedCacheMapProviderSetting setting = (ArcGISExplodedCacheMapProviderSetting) this.getMapProviderSetting();
        String path = setting.getCachePath() + "/conf.cdi";
        return path;
    }

    private String getMapName() {
        ArcGISExplodedCacheMapProviderSetting setting = (ArcGISExplodedCacheMapProviderSetting) this.getMapProviderSetting();
        if (StringUtils.isNotBlank(setting.getMapName())) {
            return setting.getMapName();
        }
        return MAPNAME;
    }
}
