package com.supermap.sample.serviceprovider;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.supermap.services.components.commontypes.Point2D;
import com.supermap.services.components.commontypes.Rectangle2D;

/**
 * <p>
 * ArcGIS Server缓存conf文件读取类  提取conf.xml和conf.cdi相关信息。
 * </p>
 * <p>
 * ArcGISXMLParseTool 提供了读取缓存目录下的conf.xml文件和conf.cdi文件相关信息的功能。
 * </p>
 * <p>
 * ArcGIS Server cache conf read class Extract conf.xml and conf.cdi.
 * </p>
 * <p>
 * ArcGISXMLParseTool provides the related function to read the conf.xml and conf.cdi infos.
 * </p>
 */
public class ArcGISXMLParseTool {
    private static DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

    private double[] resolutions;
    private double[] scales;
    private int wkid, tileCols, tileRows;
    private String cachePath;

    /**
     * <p>
     * 带参构造函数
     * </p>
     * <p>
     * The constructor with parameter
     * </p>
     * @param cache
     */
    public ArcGISXMLParseTool(String cachePath) {
        this.cachePath = cachePath;
    }

    /**
     * <p>
     * 获取地图比例尺集合
     * </p>
     * <p>
     * Get map scale set
     * </p>
     * @return 地图比例尺集合
     * <p>
     * Map scale set
     * </p>
     */
    public double[] getScales() {
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(cachePath);
            NodeList scaleDenominatorList = doc.getElementsByTagName("Scale");
            scales = new double[scaleDenominatorList.getLength()];
            for (int i = 0; i < scaleDenominatorList.getLength(); i++) {
                Node scaleDenominatorNode = scaleDenominatorList.item(i);
                scales[i] = 1 / Double.parseDouble(scaleDenominatorNode.getTextContent());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (scales == null) {
            return new double[0];
        }
        return scales;
    }

    /**
     * <p>
     * 获取地图分辨率集合
     * </p>
     * <p>
     * Get the map resolution set
     * </p>
     * @return 地图分辨率集合
     * <p>
     * Map resolution set
     * </p>
     */
    public double[] getResolutions() {
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(cachePath);
            NodeList resolutionList = doc.getElementsByTagName("Resolution");
            resolutions = new double[resolutionList.getLength()];
            for (int i = 0; i < resolutionList.getLength(); i++) {
                Node resolutionNode = resolutionList.item(i);
                resolutions[i] = Double.parseDouble(resolutionNode.getTextContent());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (resolutions == null) {
            return new double[0];
        }
        return resolutions;
    }

    /**
     * <p>
     * 获取地图WKID
     * </p>
     * <p>
     * Get the map WKID
     * </p>
     * @return 地图WKID
     * <p>
     * Map WKID
     * </p>
     */
    public int getWkid() {
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(cachePath);
            NodeList wkidList = doc.getElementsByTagName("WKID");
            Node wkidNode = wkidList.item(0);
            wkid = Integer.parseInt(wkidNode.getTextContent());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return wkid;
    }

    /**
     * <p>
     * 获取地图瓦片宽
     * </p>
     * <p>
     * Get the map tile width
     * </p>
     * @return 地图瓦片宽
     *<p>
     * Map tile width
     *</p>
     */
    public int getTileCols() {
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(cachePath);
            NodeList tileColsList = doc.getElementsByTagName("TileCols");
            Node tileColsNode = tileColsList.item(0);
            tileCols = Integer.parseInt(tileColsNode.getTextContent());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return tileCols;
    }

    /**
     * <p>
     * 获取地图瓦片高
     * </p>
     * <p>
     * Get the map tile height
     * </p>
     * @return 地图瓦片高
     * <p>
     * Map tile height
     * </p>
     */
    public int getTileRows() {
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(cachePath);
            NodeList tileRowsList = doc.getElementsByTagName("TileRows");
            Node tileRowsNode = tileRowsList.item(0);
            tileRows = Integer.parseInt(tileRowsNode.getTextContent());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return tileRows;
    }

    /**
     * <p>
     * 获取地图切片原点
     * </p>
     * <p>
     * Get the origin of map tile
     * </p>
     * @return 地图切片原点
     * <p>
     * The origin of map tile
     * </p>
     */
    public Point2D getTileOrigin() {
        double originX = 0;
        double originY = 0;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(cachePath);
            NodeList originXList = doc.getElementsByTagName("X");
            NodeList originYList = doc.getElementsByTagName("Y");
            originX = Double.parseDouble(originXList.item(0).getTextContent());
            originY = Double.parseDouble(originYList.item(0).getTextContent());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new Point2D(originX, originY);
    }

    /**
     * <p>
     * 获取地图全图范围
     * </p>
     * <p>
     * Get the map extent
     * </p>
     * @return 地图全图范围
     * <p>
     * Map extent
     * </p>
     */
    public Rectangle2D getBounds(String cdiPath) {
        double leftBottomX = 0;
        double leftBottomY = 0;
        double rightTopX = 0;
        double rightTopY = 0;
        try {
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(cdiPath);
            NodeList leftBottomXList = doc.getElementsByTagName("XMin");
            NodeList leftBottomYList = doc.getElementsByTagName("YMin");
            NodeList rightTopXList = doc.getElementsByTagName("XMax");
            NodeList rightTopYList = doc.getElementsByTagName("YMax");
            leftBottomX = Double.parseDouble(leftBottomXList.item(0).getTextContent());
            leftBottomY = Double.parseDouble(leftBottomYList.item(0).getTextContent());
            rightTopX = Double.parseDouble(rightTopXList.item(0).getTextContent());
            rightTopY = Double.parseDouble(rightTopYList.item(0).getTextContent());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new Rectangle2D(leftBottomX, leftBottomY, rightTopX, rightTopY);
    }
}
